# encoding: utf-8 
#
# L.G. de la Fraga, I. Vite Silva, and N. Cruz-Cortés,
# Euclidean distance fit of conics using differential evolution,
# Book Chapter in: Evolutionary Image Analysis and Signal Processing
# Studies in Computational Intelligence 2013,
# S. Gagnoni (Ed.)
# 2009, Springer 
#
import sys
import math

def f(a, b, x, y, t) :
	c = math.cos(t)
	s = math.sin(t)
	return( y*b*c - x*a*s + (a*a - b*b)*s*c )

def fp(a, b, x, y, t) :
	c = math.cos(t)
	s = math.sin(t)
	v = a*a - b*b
	return( -y*b*s - x*a*c - v*s*s + v*c*c )

n = len( sys.argv )
if n != 3:
	print "Args: x y"
	sys.exit(1)

a = 4
b = 2
x = float( sys.argv[1] ) 
y = float( sys.argv[2] ) 

# Inicialización:
ai_1 = math.sqrt( x*x + y*y )
if ai_1 < 1e-6 :
	ai_1 = math.pi/2.0

v = a*a - b*b

if math.fabs(y) < b and math.fabs(x) <= v/a :
	if math.fabs(y) < 1e-6 :
		ai_1 = math.pi/2.0
	else :
		p = math.fabs(x)/(1.0 + math.fabs(y)/ai_1)
		if x < 0 :
			p = -p
		ai_1 = math.atan2( a*y, b*(x-p) )
else :
	ai_1 = math.atan2( a*y, b*x )


imax = 30

print "0", ai_1

i=1
while i <= imax :
	ai = ai_1 - f(a, b, x, y, ai_1)/fp(a, b, x, y, ai_1)

	print i, ai

	if abs(ai-ai_1) < 1e-8 :
		break

	ai_1 = ai

	i += 1

x = a*math.cos( ai )
y = b*math.sin( ai )

print ai, x, y
